/***************************************************************************//**
 *   @file   Main.c
 *   @brief  Implementation of the program's main function.
 *   @author Dan Nechita
********************************************************************************
 * Copyright 2012(c) Analog Devices, Inc.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of Analog Devices, Inc. nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *  - The use of this software may or may not infringe the patent rights
 *    of one or more patent holders.  This license does not release you
 *    from the requirement that you obtain separate licenses from these
 *    patent holders to use this software.
 *  - Use of the software either in source or binary form, must be run
 *    on or directly connected to an Analog Devices Inc. component.
 *
 * THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
********************************************************************************
 *   SVN Revision: 776
*******************************************************************************/

/******************************************************************************/
/***************************** Option Bytes ***********************************/
/******************************************************************************/
#pragma location = "OPTBYTE"
__root const unsigned char opbyte0 = 0x00; // Watchdog timer settings.
#pragma location = "OPTBYTE"
__root const unsigned char opbyte1 = 0xFD; // Low voltage detector settings.
#pragma location = "OPTBYTE"
__root const unsigned char opbyte2 = 0xF8; // Flash op. mode and osc. settings.
#pragma location = "OPTBYTE"
__root const unsigned char opbyte3 = 0x04; // On-chip Debug Option Byte.

/******************************************************************************/
/***************************** Include Files **********************************/
/******************************************************************************/
#include "YRDKRL78G13.h"    // YRDKRL78G13 definitions.
#include "ST7579.h"         // ST7579 definitions.
#include "TIME.h"           // TIME definitions.
#include "AD569X.h"         // AD569X definitions.

/******************************************************************************/
/************************** Macros and definitions ****************************/
/******************************************************************************/
#define TABLE_SIZE  256

/******************************************************************************/
/************************** Variables Declarations ****************************/
/******************************************************************************/

const unsigned short sineTable[TABLE_SIZE] = {
2048, 2098, 2148, 2198, 2248, 2298, 2348, 2398, 2447, 2496, 2545, 2594, 2642, 
2690, 2737, 2784, 2831, 2877, 2923, 2968, 3013, 3057, 3100, 3143, 3185, 3226, 
3267, 3307, 3346, 3385, 3423, 3459, 3495, 3530, 3565, 3598, 3630, 3662, 3692, 
3722, 3750, 3777, 3804, 3829, 3853, 3876, 3898, 3919, 3939, 3958, 3975, 3992, 
4007, 4021, 4034, 4045, 4056, 4065, 4073, 4080, 4085, 4089, 4093, 4094, 4095, 
4094, 4093, 4089, 4085, 4080, 4073, 4065, 4056, 4045, 4034, 4021, 4007, 3992, 
3975, 3958, 3939, 3919, 3898, 3876, 3853, 3829, 3804, 3777, 3750, 3722, 3692, 
3662, 3630, 3598, 3565, 3530, 3495, 3459, 3423, 3385, 3346, 3307, 3267, 3226, 
3185, 3143, 3100, 3057, 3013, 2968, 2923, 2877, 2831, 2784, 2737, 2690, 2642, 
2594, 2545, 2496, 2447, 2398, 2348, 2298, 2248, 2198, 2148, 2098, 2048, 1997, 
1947, 1897, 1847, 1797, 1747, 1697, 1648, 1599, 1550, 1501, 1453, 1405, 1358, 
1311, 1264, 1218, 1172, 1127, 1082, 1038,  995,  952,  910,  869,  828,  788,
 749,  710,  672,  636,  600,  565,  530,  497,  465,  433,  403,  373,  345, 
 318,  291,  266,  242,  219,  197,  176,  156,  137,  120,  103,   88,   74, 
  61,   50,   39,   30,   22,   15,   10,    6,    2,    1,    0,    1,    2,
   6,   10,   15,   22,   30,   39,   50,   61,   74,   88,  103,  120,  137, 
 156,  176,  197,  219,  242,  266,  291,  318,  345,  373,  403,  433,  465, 
 497,  530,  565,  600,  636,  672,  710,  749,  788,  828,  869,  910,  952, 
 995, 1038, 1082, 1127, 1172, 1218, 1264, 1311, 1358, 1405, 1453, 1501, 1550, 
1599, 1648, 1697, 1747, 1797, 1847, 1897, 1947, 1997};

const unsigned short squareTable[TABLE_SIZE] = {
   0,  146,  292,  438,  585,  731,  877, 1023, 1170, 1316, 1462, 1608, 1755, 
1901, 2047, 2193, 2340, 2486, 2632, 2778, 2925, 3071, 3217, 3363, 3510, 3656, 
3802, 3948, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 
4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 
4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 
4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 
4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 
4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 
4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 
4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 3948, 
3802, 3656, 3510, 3363, 3217, 3071, 2925, 2778, 2632, 2486, 2340, 2193, 2047, 
1901, 1755, 1608, 1462, 1316, 1170, 1023,  877,  731,  585,  438,  292,  146,   
   0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   0,    0,    0,    0,    0,    0,    0,    0,    0};

unsigned char point = 0; 

/***************************************************************************//**
* @brief Draws the ADI logo and displays the name of ADI component.
*
* @param componentName - The name of the ADI component.
*
* @return None.
*******************************************************************************/
void ADI_Component(unsigned char* string)
{
    const unsigned char adiComponent[7] =
    {
        0xFE, 0xFE, 0x82, 0xC6, 0xEE, 0xFE, 0xFE 
    };
    ST7579_Image(0, 0, adiComponent, 7, 8);
    ST7579_String(0, 9, string, 0);
}

/***************************************************************************//**
 * @brief Main function.
 *
 * @return None.
*******************************************************************************/
void main(void)
{   
    unsigned short readBackVal = 0;
    float          outVoltage  = 0;
    
    /* Initialize YRDKRL78G13. */
    YRDKRL78G13_Init();
    
    /* Enable interrupts. */
    __enable_interrupt();
        
    /* Initialize timer. */
    TIME_Init();
    
    /* Initialize the ST7579 Display. */
    ST7579_Init();
    
    /* Initialize AD569X device. */
    if( AD569X_Init(AD5696R, AD569X_A1_HIGH, AD569X_A0_LOW))
    {
        ADI_Component("AD569X OK");
    }
    else
    {
        ADI_Component("AD569X Err");
    }
    /* Display the menu on the LCD. */
    ST7579_String(2, 0, "DAC A [raw]:", 0);
    ST7579_String(4, 0, "DAC B [V]:", 0);
    ST7579_String(6, 0, "DAC C: sine", 0);   
    ST7579_String(7, 0, "DAC D: trapezoid", 0);   
    /* Enable internal voltage reference. */
    AD569X_InternalVoltageReference(AD569X_INT_REF_ON);
    /* Power-up all channels. */
    AD569X_PowerMode(AD569X_ADDR_DAC_A, AD569X_PD_NORMAL);
    AD569X_PowerMode(AD569X_ADDR_DAC_B, AD569X_PD_NORMAL);
    AD569X_PowerMode(AD569X_ADDR_DAC_C, AD569X_PD_NORMAL);
    AD569X_PowerMode(AD569X_ADDR_DAC_D, AD569X_PD_NORMAL);
    TIME_DelayUs(5);
    /* Reset the device. */
    AD569X_Reset(AD569X_RST_ZERO_SCALE);
    /* Set the channel A output voltage to 0.45 V and read back the raw value
       written to DAC A register.*/
    AD569X_SetVoltage(AD569X_ADDR_DAC_A, 0.45, 2.5);
    readBackVal = AD569X_ReadBack(AD569X_ADDR_DAC_A);
    ST7579_HexNumber(3, 0, readBackVal, 0);
    /* Set the channel B output voltage to 1.40 V and diplay on the LCD the 
       nearest value of the voltage that AD569X can output. */
    outVoltage = AD569X_SetVoltage(AD569X_ADDR_DAC_B, 1.25, 2.5);
    ST7579_FloatNumber(5, 0, outVoltage, 3, 0);   
    /* Mask channel A and channel B in order to ingnore LDAC transitions. */
    AD569X_SetInputRegister(AD569X_CMD(AD569X_CMD_LDAC_MASK) |
                             AD569X_ADDR_DAC_A | 
                             AD569X_ADDR_DAC_B);
    while(1)
    {
        AD569X_LDAC_HIGH;
        if(point >= TABLE_SIZE)
        {
            point = 0;
        }
        /* Load channel C and channel D with values from the tables. These 
        values will be outputted simultaneously when LDAC is asserted. */
        AD569X_WriteFunction(AD569X_CMD_WR_INPUT_N, 
                             AD569X_ADDR_DAC_C, 
                             sineTable[point] << 4);
        AD569X_WriteFunction(AD569X_CMD_WR_INPUT_N, 
                             AD569X_ADDR_DAC_D, 
                             squareTable[point] << 4);
        /* Assert LDAC. */
        AD569X_LDAC_LOW;
        point++;
        TIME_DelayMs(5);
    }
}
